@@ -25,6 +25,7 @@
 #include "brpc/channel_base.h"
 #include "brpc/controller.h"
 #include "brpc/proto_base.pb.h"
+#include "brpc/pb_compat.h"
 
 namespace apache {
 namespace thrift {
@@ -83,27 +84,27 @@ public:
   
     // implements Message ----------------------------------------------
   
-    ThriftFramedMessage* New() const;
+    ThriftFramedMessage* New() const PB_319_OVERRIDE;
 #if GOOGLE_PROTOBUF_VERSION >= 3006000
     ThriftFramedMessage* New(::google::protobuf::Arena* arena) const override;
 #endif
-    void CopyFrom(const ::google::protobuf::Message& from);
-    void MergeFrom(const ::google::protobuf::Message& from);
+    void CopyFrom(const ::google::protobuf::Message& from) PB_321_OVERRIDE;
+    void MergeFrom(const ::google::protobuf::Message& from) override;
     void CopyFrom(const ThriftFramedMessage& from);
     void MergeFrom(const ThriftFramedMessage& from);
-    void Clear();
-    bool IsInitialized() const;
+    void Clear() override;
+    bool IsInitialized() const override;
   
     int ByteSize() const;
     bool MergePartialFromCodedStream(
-        ::google::protobuf::io::CodedInputStream* input);
+        ::google::protobuf::io::CodedInputStream* input) PB_310_OVERRIDE;
     void SerializeWithCachedSizes(
-        ::google::protobuf::io::CodedOutputStream* output) const;
-    ::google::protobuf::uint8* SerializeWithCachedSizesToArray(::google::protobuf::uint8* output) const;
-    int GetCachedSize() const { return ByteSize(); }
+        ::google::protobuf::io::CodedOutputStream* output) const PB_310_OVERRIDE;
+    ::google::protobuf::uint8* SerializeWithCachedSizesToArray(::google::protobuf::uint8* output) const PB_310_OVERRIDE;
+    int GetCachedSize() const override { return ByteSize(); }
 
 protected:
-    ::google::protobuf::Metadata GetMetadata() const;
+    ::google::protobuf::Metadata GetMetadata() const override;
 
 private:
     void SharedCtor();
