@@ -167,12 +167,17 @@ public:
     // Returns address of the inserted value, NULL on error.
     mapped_type* insert(const key_type& key, const mapped_type& value);
 
+    // Insert a pair of {key, value}. If size()*100/bucket_count() is
+    // more than load_factor(), a resize() will be done.
+    // Returns address of the inserted value, NULL on error.
+    mapped_type* insert(const std::pair<key_type, mapped_type>& kv);
+
     // Remove |key| and the associated value
     // Returns: 1 on erased, 0 otherwise.
-    // Remove all items. Allocated spaces are NOT returned by system.
     template <typename K2>
     size_t erase(const K2& key, mapped_type* old_value = NULL);
 
+    // Remove all items. Allocated spaces are NOT returned by system.
     void clear();
 
     // Remove all items and return all allocated spaces to system.
